-- zboards.e
-- ZIG board functions / board/world properties

include zutil.e
include machine.e

global constant
    -- World property names and accompanying constants
    WBP_STRING = 1, WBP_BOARD = 2, WBP_NUMBER = 3,
    WBP_PROMPT = 4, WBP_VALTYPE = 5, WBP_PALETTE = 6, WBP_CHARSET = 7,
    
    W_PROPS = {{"World Name", 1}, {"Title Board", 2}, {"Starting Board", 2}},
               
    WP_NAME = 1, WP_TBOARD = 2, WP_SBOARD = 3,

    -- Board property names and accompanying constants
    B_PROPS = {{"Board Name", 1}, {"Board to North", 2}, {"Board to South", 2},
               {"Board to West", 2}, {"Board to East", 2}, {"Palette", 6},
               {"Character Set", 7}},
               
    BP_NAME = 1, BP_BN = 2, BP_BS = 3, BP_BW = 4, BP_BE = 5, BP_PAL = 6,
    BP_CHARSET = 7,
    
    BOARD_ELEMS = 25 * (60 * (3 * 3))
    
    
global sequence
    Boards, OneBoard, PropData, WorldProps, BoardProps

global atom CurBoard


-- return index for use with Boards[BoardNum][BoardI(..)]
global function BoardI(atom x, atom y, atom l, atom i)
    atom ret

    -- this was a MAJOR headache to figure out...
    -- ...but now worlds will load faster!    
    ret = ((x - 1) * 540) + ((y - 1) * 9) + ((l - 1) * 3) + (i - 1)
    if ret = 0 then
        ret = 1
    end if
    
    return(ret)
end function

global procedure DrawObjects(atom Board)
    atom oi
    
    for z = 1 to length(Objects) do
        if Objects[z][1][OBJ_BOARD_NUM] = Board then
            if Objects[z][1][OBJ_LAYER] != 3 then
                if Boards[Board][BoardI(Objects[z][1][OBJ_X], Objects[z][1][OBJ_X], Objects[z][1][OBJ_LAYER] + 1, CHAR)] = 0 then
                    display_text_image(
                        {Objects[z][1][OBJ_X], Objects[z][1][OBJ_Y]},
                        {{Objects[z][1][OBJ_CHAR], Objects[z][1][OBJ_COLOR]}})
                end if
            else
                display_text_image(
                    {Objects[z][1][OBJ_X], Objects[z][1][OBJ_Y]},
                    {{Objects[z][1][OBJ_CHAR], Objects[z][1][OBJ_COLOR]}})
            end if
        end if
    end for
    
    return
    
    -- old code down here
    
    if length(Objects) != 0 then
        for x = 1 to 25 do            
            for y = 1 to 80 do
                for z = 1 to 3 do
                    if ObjectPosTable[x][y][z] != 0 then
                        if z != 3 then
                            if Boards[Board][BoardI(x, y, z + 1, CHAR)] = 0 then
                                oi = ObjectPosTable[x][y][z]
                                display_text_image(
                                {Objects[oi][1][OBJ_X], Objects[oi][1][OBJ_Y]},
                                {{Objects[oi][1][OBJ_CHAR], Objects[oi][1][OBJ_COLOR]}}
                                )
                            end if
                        else                            
                            oi = ObjectPosTable[x][y][z]
                            display_text_image({Objects[oi][1][OBJ_X], Objects[oi][1][OBJ_Y]}, {Objects[oi][1][OBJ_CHAR], Objects[oi][1][OBJ_COLOR]})
                        end if    
                    end if
                end for
            end for
        end for
    end if    
end procedure

global procedure DrawBoard1L(atom BoardW, atom X, sequence Layers)
    atom rY
    sequence D1, Draw, Coords
        
    D1 = repeat(0, 120)
    Draw = {D1}
    Coords = {X, 1}
  
    for Y = 1 to 120 by 2 do
            rY = (Y / 2) + .5
            for z = 1 to 3 do
                if Boards[BoardW][BoardI(X, rY, z, CHAR)] != NULL then
                    if Layers[z] != OFF then
                        Draw[1][Y] = Boards[BoardW][BoardI(X, rY, z, CHAR)]
                        Draw[1][Y + 1] = Boards[BoardW][BoardI(X, rY, z, COL)]
                    end if
                end if
            end for
        end for

    display_text_image(Coords, Draw)
end procedure

global procedure DrawBoard(atom BoardW, sequence Layers)
    atom rY
    sequence D1, Draw, Coords
        
    D1 = repeat(0, 120)
    Draw = repeat(D1, 25)
    Coords = repeat(1, 2)

    for X = 1 to 25 do
        for Y = 1 to 120 by 2 do
            rY = (Y / 2) + .5
            for z = 1 to 3 do
                if Boards[BoardW][BoardI(X, rY, z, CHAR)] != NULL then
                    if Layers[z] = ON then
                        Draw[X][Y] = Boards[BoardW][BoardI(X, rY, z, CHAR)]
                        Draw[X][Y + 1] = Boards[BoardW][BoardI(X, rY, z, COL)]
                    end if
                end if
            end for
        end for
    end for
    
    display_text_image(Coords, Draw)  
    --DrawObjects() for editor
end procedure

global procedure DrawTile(atom Board, atom X, atom Y, sequence Layers)
    atom oi
    sequence Dr
    Dr = {0, 0}
        
    for z = 1 to 3 do
        if Boards[Board][BoardI(X, Y, z, CHAR)] != 0 and Layers[z] = ON then
            Dr[1] = Boards[Board][BoardI(X, Y, z, CHAR)]
            Dr[2] = Boards[Board][BoardI(X, Y, z, COL)]
        end if
    end for
    
    display_text_image({X, Y}, {Dr})
    
    for z = 1 to 3 do
        if ObjectPosTable[X][Y][z] != 0 then
            if z != 3 then
                if Boards[Board][BoardI(X, Y, z + 1, 1)] = 0 then
                    oi = ObjectPosTable[X][Y][z]
                    display_text_image(
                        {Objects[oi][1][OBJ_X], Objects[oi][1][OBJ_Y]},
                        {{Objects[oi][1][OBJ_CHAR], Objects[oi][1][OBJ_COLOR]}})
                end if
            else                            
                oi = ObjectPosTable[X][Y][z]
                display_text_image({Objects[oi][1][OBJ_X], Objects[oi][1][OBJ_Y]}, {Objects[oi][1][OBJ_CHAR], Objects[oi][1][OBJ_COLOR]})
            end if    
        end if
    end for
end procedure

global procedure DrawBoard_Ed(atom B, sequence L)
    DrawBoard(B, L)
    DrawObjects(B)
end procedure

global procedure DrawObjectsInGame(sequence Runs, sequence Olds, atom Board)
    atom z
    sequence Dr
    
    Dr = {0,0}
    
    bk_color(0)
    
    if length(Runs) != 0 then
        for ax = 1 to length(Runs) do
            z = Runs[ax]
            if Objects[z][1][OBJ_DISPMODE] = 1 then
                if Olds[ax][1] != 0 then
                    if Olds[ax][1] != Objects[z][1][OBJ_X]
                    or Olds[ax][2] != Objects[z][1][OBJ_Y]
                    or Olds[ax][3] != Objects[z][1][OBJ_LAYER] then
                        DrawTile(Board, Olds[ax][1], Olds[ax][2], {1, 1, 1})
                    end if
                end if
                    
                if Olds[ax][4] = TRUE then
                    if Boards[Board][
                            BoardI(Objects[z][1][OBJ_X],
                                Objects[z][1][OBJ_Y],
                                3, CHAR)] = 0 then
                        Dr[1] = Objects[z][1][OBJ_CHAR]
                        Dr[2] = Objects[z][1][OBJ_COLOR]
                        display_text_image({Objects[z][1][OBJ_X],
                                Objects[z][1][OBJ_Y]}, {Dr})
                    end if
                end if
            end if
        end for
    end if
end procedure

